Package: Seqinfo
Title: A simple S4 class for storing basic information about a
        collection of genomic sequences
Description: The Seqinfo class stores the names, lengths, circularity flags,
	and genomes for a particular collection of sequences. These sequences
	are typically the chromosomes and/or scaffolds of a specific genome
	assembly of a given organism. Seqinfo objects are rarely used as
	standalone objects. Instead, they are used as part of higher-level
	objects to represent their seqinfo() component. Examples of such
	higher-level objects are GRanges, RangedSummarizedExperiment, VCF,
	GAlignments, etc... defined in other Bioconductor infrastructure
	packages.
biocViews: Infrastructure, DataRepresentation, GenomeAssembly,
        Annotation, GenomeAnnotation
URL: https://bioconductor.org/packages/Seqinfo
BugReports: https://github.com/Bioconductor/Seqinfo/issues
Version: 1.3.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: person("Hervé", "Pagès", role=c("aut", "cre"),
		  email="hpages.on.github@gmail.com",
		  comment=c(ORCID="0009-0002-8272-4522"))
Depends: methods, BiocGenerics
Imports: stats, S4Vectors (>= 0.47.6), IRanges
Suggests: GenomeInfoDb, GenomicRanges, BSgenome, GenomicFeatures,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        BSgenome.Hsapiens.UCSC.hg38, BSgenome.Celegans.UCSC.ce2, RUnit,
        knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
Collate: utils.R rankSeqlevels.R seqinfo.R sortSeqlevels.R
        Seqinfo-class.R seqlevelsInUse.R GenomeDescription-class.R
        zzz.R
git_url: https://git.bioconductor.org/packages/Seqinfo
git_branch: devel
git_last_commit: 0dcd005
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:05:20 UTC; biocbuild
Author: Hervé Pagès [aut, cre] (ORCID: <https://orcid.org/0009-0002-8272-4522>)
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
