[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**Seqinfo** is an R/Bioconductor package that implements a simple S4 class for storing basic information about a collection of genomic sequences.

See https://bioconductor.org/packages/Seqinfo for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

